#include "widget.h"
#include "ui_widget.h"
#include<iostream>
#include<QMessageBox>
#include <QTimer>

Widget::Widget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::Widget)
{
    ui->setupUi(this);


    this->valMem=0;
    m_settings= new SettingsDialog();
    m_serial = new QSerialPort(this);
    mCheckboxes = new QVector<QCheckBox*>;

  for(int i = 1;i<9;i++)
    this->changeColor(i,QColor(Qt::lightGray));

  ui->pushButtonDisconnect->setEnabled(false);

  mCheckboxes->append(ui->checkBox1);
  mCheckboxes->append(ui->checkBox2);
  mCheckboxes->append(ui->checkBox3);
  mCheckboxes->append(ui->checkBox4);
  mCheckboxes->append(ui->checkBox5);
  mCheckboxes->append(ui->checkBox6);
  mCheckboxes->append(ui->checkBox7);
  mCheckboxes->append(ui->checkBox8);
 connect(ui->pushButton,SIGNAL(pressed()),this,SLOT(close()));

 for (int i = 0; i < mCheckboxes->count(); i++)
 {
     connect(mCheckboxes->at(i),SIGNAL(released()),this,SLOT(communicateUSB()));
 }
 connect(ui->pushButtonConf,SIGNAL(pressed()),m_settings,SLOT(show()));
 connect(ui->pushButtonConnect,SIGNAL(pressed()),this,SLOT(openSerialPort()));
 connect(ui->pushButtonDisconnect,SIGNAL(pressed()),this,SLOT(closeSerialPort()));

 connect(m_serial,SIGNAL(errorOccurred()),this,SLOT(handleError()));
 connect(m_serial,SIGNAL(readyRead()),this,SLOT(updateLED()));

}


void Widget::communicateUSB()
{
 //   QCheckBox* cb = qobject_cast<QCheckBox*>(sender());
 //   int idx = mCheckboxes->indexOf(cb);
    unsigned char val = this->getSwitch();
    //this->setLED(val); // uniquement pour tester !!
    std::cout<<"valeur à tranmettre"<<((int) val)<<std::endl;

    // PASCAL Ecriture des données sur le port série

    unsigned char comp = val^this->valMem;

    int diff = val-this->valMem;

    char *mess = new char[6] ;

    if(diff>0)
        mess[0]='S';
    else
        mess[0]='R';

    this->int2hex(mess+1,comp);

    mess[5]='\n';
    std::cout<<" message "<<mess<<std::endl;

    this->valMem=val;

    m_serial->write(mess,6);

/*    char mot[6]="O00FF\n";
    m_serial->write(mot,6); */

}

void Widget::updateLED()
{
    const QByteArray data = m_serial->readAll();


    //std::cout<<"Chaine reçue "<<data.constData()<<std::endl;

     // PASCAL  lecture sur le port série
     unsigned int intVal= this->hex2int(data);
    // std::cout<<"val int"<<intVal<<std::endl;
     intVal >>= 8; // 8 MSB pour les leds
    // std::cout<<"val int"<<intVal<<std::endl;
    this->setLED((unsigned char) intVal);
}


unsigned int Widget::hex2int(QByteArray data)
{
    int ptr=4;
    char quartet;
    unsigned int val=0;
    while (ptr--) {
        quartet=(data[3-ptr]);
        val+=(((quartet>'9')?(quartet&0x5f)-'7':quartet-'0')<<(ptr*4));
    }
    return val;
}


char * Widget::int2hex(char * Buffer,unsigned int val){
  // conversion en chaine ascii hexa d'un nombre 16 bits
  // on suppose que buffer est >= 5 caractères
  int quartet,ptr=4;
  ptr=4;
  while (ptr--) {
    quartet=(val>>(4*ptr))&0x0F;
    Buffer[3-ptr]=(quartet>9)?quartet+'7':quartet+'0';
  }
  Buffer[4]=0;
  return Buffer;
}


unsigned char Widget::getSwitch()
{
 unsigned char tmp = 0 ;
 tmp = ui->checkBox1->isChecked();
 tmp += ui->checkBox2->isChecked()*2;
 tmp += ui->checkBox3->isChecked()*4;
 tmp += ui->checkBox4->isChecked()*8;
 tmp += ui->checkBox5->isChecked()*16;
 tmp += ui->checkBox6->isChecked()*32;
 tmp += ui->checkBox7->isChecked()*64;
 tmp += ui->checkBox8->isChecked()*128;

 return tmp;
}

void Widget::changeColor(int num, QColor couleur)
{
    QToolButton * tmpButton=nullptr;
    switch(num)
    {
       case 1 :
          tmpButton =ui->toolButton1;
          break;

       case 2 :
          tmpButton =ui->toolButton2;
          break;

       case 3 :
          tmpButton =ui->toolButton3;
          break;

       case 4 :
          tmpButton =ui->toolButton4;
          break;

       case 5 :
          tmpButton =ui->toolButton5;
          break;

       case 6 :
          tmpButton =ui->toolButton6;
          break;

       case 7 :
          tmpButton =ui->toolButton7;
          break;

      case 8 :
         tmpButton =ui->toolButton8;
         break;

    }

    if(couleur.isValid()) {
       QString qss = QString("background-color: %1").arg(couleur.name());
       tmpButton->setStyleSheet(qss);
    }

    /*
    tmpButton->setAutoFillBackground(true);
    QPalette pal std::cout<<"valeur à tranmettre"<<((int) val)<<std::endl;
ette = tmpButton->palette();
    palette.setColor(QPalette::Base, couleur);
    tmpButton->setPalette(palette);
    tmpButton->show();
*/

}


void Widget::setLED(unsigned char val)
{
    unsigned char rang=8;
    while (rang--) {
        this->changeColor(rang+1,(val&(1<<rang))?QColor(Qt::red):QColor(Qt::lightGray));
    }
   /* for (int i=0;i<8;i++) {

        if(val%2==1)
             this->changeColor(i+1,QColor(Qt::red));
        else
           this->changeColor(i+1,QColor(Qt::lightGray));

        val = val/2;

    }
*/
}

void Widget::handleError(QSerialPort::SerialPortError error)
{
    if (error == QSerialPort::ResourceError) {
        QMessageBox::critical(this, tr("Critical Error"), m_serial->errorString());
        closeSerialPort();
    }
}

void Widget::handleSerialPortOpen()
{
     char mot[7] = "O00FF\n";
     m_serial->write(mot,6);
     mot[0]='R';
     m_serial->write(mot,6);
     m_serial->write(mot,6);
}


void Widget::openSerialPort()
{
    const SettingsDialog::Settings p = m_settings->settings();
    m_serial->setPortName(p.name);
    m_serial->setBaudRate(p.baudRate);
    m_serial->setDataBits(p.dataBits);
    m_serial->setParity(p.parity);
    m_serial->setStopBits(p.stopBits);
    m_serial->setFlowControl(p.flowControl);
    if (m_serial->open(QIODevice::ReadWrite)) {
        /* m_console->setEnabled(true);
         m_console->setLocalEchoEnabled(p.localEchoEnabled);*/
         ui->pushButtonConnect->setEnabled(false);
         ui->pushButtonDisconnect->setEnabled(true);
         ui->pushButtonConf->setEnabled(false);
         showStatusMessage(tr("Connected to %1 : %2, %3, %4, %5, %6")
                           .arg(p.name).arg(p.stringBaudRate).arg(p.stringDataBits)
                           .arg(p.stringParity).arg(p.stringStopBits).arg(p.stringFlowControl));

        QTimer::singleShot(1000, this, SLOT(handleSerialPortOpen()));

    } else {
        QMessageBox::critical(this, tr("Error"), m_serial->errorString());
        showStatusMessage(tr("Open error"));
    }
}

void Widget::closeSerialPort()
{
    if (m_serial->isOpen())
        m_serial->close();
    ui->pushButtonConf->setEnabled(true);
    ui->pushButtonConnect->setEnabled(true);
    ui->pushButtonDisconnect->setEnabled(false);
    showStatusMessage(tr("Disconnected"));
}

void Widget::showStatusMessage(const QString &message)
{
    ui->labelMessage->setText(message);
}

Widget::~Widget()
{
    delete ui;
}

