/************************************************************************
 * ctrl_tty : contrôle des E/S TOR d'une carte Arduino à travers
 * une liaison série.
 * Objectif : interfaçage de carte de TP pour télé-TP
 * 
 * Remarques, debug, etc : pascal.varoqui@ens-paris-saclay.fr
 * 
 * 23 mar 2020 : version initiale
 * 
 ************************************************************************/

#define BAUDRATE 115200 
#define LBUFF 10

// liste des 16 E/S associées aux bits manipulés.
// ( à adapter selon carte utilisée)
const int bitnum[]={A0,A1,A2,A3,A4,A5,2,3,4,5,6,7,8,9,10,11};

//------------------------------------------------------------------
unsigned int hex2int(char * hexval) {
  // chaine de 4 caractères ascii hexa convertie en valeur bin
  int ptr=4;
  char quartet;
  unsigned int val=0;
  while (ptr--) {
    quartet=(hexval[3-ptr]);
    val+=(((quartet>'9')?(quartet&0x5f)-'7':quartet-'0')<<(ptr*4));
  }
  return val;
}
//------------------------------------------------------------------
unsigned int lecture_bin(){
  // valeurs binaires des bits E/S rassemblées sur 2 octets
  int ptr=16;
  unsigned int val=0;
  while (ptr--) {
    val|=digitalRead(bitnum[ptr])?(1<<ptr):0;
  }
  return val;
}
//-----------------------------------------------------------------
char * int2hex(char * Buffer,unsigned int val) {
  // conversion en chaine ascii hexa d'un nombre 16 bits
  // on suppose que buffer est >= 5 caractères
  int quartet,ptr=4;
  ptr=4;
  while (ptr--) {
    quartet=(val>>(4*ptr))&0x0F;
    Buffer[3-ptr]=(quartet>9)?quartet+'7':quartet+'0';
  }
  Buffer[4]=0;
  return Buffer;
}
//------------------------------------------------------------------
void Traitement_Rx(char * Buffer) {
  // traitement des commandes reçues :
  // O : définition du sens des bits d'E/S
  // S : mise à un des bits désignés
  // R : mise à 0 des bits désignés
  unsigned int val=hex2int(Buffer+1);
  int ptr=16;
  switch (Buffer[0]) {
    case 'O':
    case 'o':
      while (ptr--) {
        if (val & (1<<ptr)) {
          pinMode(bitnum[ptr],OUTPUT);
        } else {
          pinMode(bitnum[ptr],INPUT);
        }
      }
      break;
    case 'S':
    case 's':
      while (ptr--) {
        if (val & (1<<ptr)) {
          digitalWrite(bitnum[ptr],HIGH);
        }
      }
      break;
    case 'R':
    case 'r':
      while (ptr--) {
        if (val & (1<<ptr)) {
          digitalWrite(bitnum[ptr],LOW);
        }
      }
      break;
    default:
      Serial.println("Erreur");
  }
}
//------------------------------------------------------------------
void setup() {
  Serial.begin(BAUDRATE); 
}
//------------------------------------------------------------------
void loop() {
  char Buffer[LBUFF+1];
  char Buffer_tx[5];
  unsigned int etat_ES,etat_ES_prec,ptr=0;

  while (1) { // boucle infinie principale
    if (Serial.available()) { // construction de la chaine rx
      Buffer[ptr++]=(char)Serial.read(); // 1 char à la fois
      if ((ptr>=LBUFF) or (Buffer[ptr-1]=='\n')) {
        Buffer[ptr]=0;
        Traitement_Rx(Buffer);
        ptr=0;
        Serial.println(int2hex(Buffer_tx,etat_ES_prec=lecture_bin()));
      }
    } 
    if (etat_ES_prec!=(etat_ES=lecture_bin())) {
      Serial.println(int2hex(Buffer_tx,etat_ES));
      etat_ES_prec=etat_ES;
    }
  }
}
